//////////////////////////////////////////////////////////////////////////////////
// 
// DSairLite LogManager
// Copyright(C)'2024- DesktopStation
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//

#include <arduino.h>
#include <EEPROM.h>
#include "pico/stdlib.h"
#include <LittleFS.h>
#include "LogManager.h"
#include "DSVersion.h"


//初期化する
LogManager::LogManager()
{
	callback_dscommand = NULL;
	callback_dscore = NULL;

	memset(u4_logdata, 0, sizeof(u4_logdata));

	Clear();





}

void LogManager::Clear()
{
	//BIDI CUTOFF
	u2_logcounter = 0;

	for( int i = 0; i < LOGSIZE; i++)
	{
		u4_logdata[i] = 0;
	}
	
	u2_locaddress_playing = 0;
	u4_wait_playing = 0;
	u4_FlagFunction = 0;
	
	u2_locaddress = 65535;
	u4_currenttime = 0;
	u2_playcounter = 0;
	u1_state = LOGSTATE_IDLE;


}

void LogManager::Reset()
{
	
	
	
}

void LogManager::Save()
{

	LittleFS.begin();
	
	File f = LittleFS.open("/robot_log.bin", "w");
	
	if (!f)
	{
	    Serial.println("@FS,file open failed for saving");
	}
	else
	{

		uint8_t buffer[4];

		buffer[0] = u2_logcounter & 0xFF;
		buffer[1] = (u2_logcounter >> 8) & 0xFF;
		buffer[2] = u2_locaddress & 0xFF;
		buffer[3] = (u2_locaddress >> 8) & 0xFF;

		f.write(buffer, 4);

		for( int i = 0; i < LOGSIZE; i++)
		{
			buffer[0] = u4_logdata[i] & 0xFF;
			buffer[1] = (u4_logdata[i] >> 8) & 0xFF;
			buffer[2] = (u4_logdata[i] >> 16) & 0xFF;
			buffer[3] = (u4_logdata[i] >> 24) & 0xFF;

			f.write(buffer, 4);
		}

	    Serial.println("@FS,saved");
	}

	f.close();
	
	LittleFS.end();

}

bool LogManager::Exist()
{
	bool aRet = false;
	

	LittleFS.begin();
	
	File f = LittleFS.open("/robot_log.bin", "r");
	if (!f)
	{
	    Serial.println("@FS,file open failed");
	}
	else
	{
		//ファイルは存在
		aRet = true;
	}
	
	f.close();

	LittleFS.end();

	return aRet;
}



void LogManager::Load()
{

	LittleFS.begin();
	
	File f = LittleFS.open("/robot_log.bin", "r");
	
	if (!f)
	{
	    Serial.println("@FS,file open failed for loading");
	}
	else
	{
		//読み込む前に初期化する
		Clear();

	    Serial.println("@FS,load start");
		Serial.print("@FS,");
		Serial.println(f.size());

		if( f.size() >= (LOGSIZE * 4 + 2 * 2))
		{
			//ファイル内容の読み出し
			uint8_t buffer[4];

			f.read(buffer, 4);

			u2_logcounter = buffer[0] + (buffer[1] << 8);
			u2_locaddress = buffer[2] + (buffer[3] << 8);

			for( int i = 0; i < LOGSIZE; i++)
			{
				f.read(buffer, 4);
				u4_logdata[i] = buffer[0] + (buffer[1] << 8) + (buffer[2] << 16) + (buffer[3] << 24);
			}

			Serial.println("@FS,loaded");
		}
		else
		{
			Serial.println("@FS,file size is wrong");
		}


	}
	
	f.close();

	LittleFS.end();	

}

LOG_CMD LogManager::Main()
{
	//Interval task (100ms)
	LOG_CMD aCmd;
	
	aCmd.function = "";
	aCmd.arg0 = 0;
	aCmd.arg1 = 0;
	aCmd.arg2 = 0;
	
	
	switch(u1_state)
	{
	case LOGSTATE_PLAY:
		//逐次実行
		aCmd = RunInterpreter();
		
		break;
		
	case LOGSTATE_LOG:
		
		break;
		
	case LOGSTATE_IDLE:
		
		break;
		
	}
	
	return aCmd;
}

LOG_CMD LogManager::RunInterpreter()
{
	uint32_t aData = 0;
	uint32_t aParam1 = 0;
	uint32_t aParam2 = 0;
	LOG_CMD aCmd;
	
	aCmd.function = "";
	aCmd.arg0 = 0;
	aCmd.arg1 = 0;
	aCmd.arg2 = 0;
	
	//データ取得
	aData = GetLog(u2_playcounter);
	
	if( (aData == 0) || (u2_logcounter <= u2_playcounter))
	{
		//最初に戻る
		u2_playcounter = 0;
		
	}
	else
	{
		//CMD
		
		if( (u4_wait_playing != 0) && (to_ms_since_boot(get_absolute_time()) <= u4_wait_playing))
		{
			//ウェイト
			
			
		}
		else
		{
			//ウェイト初期化
			u4_wait_playing = 0;
			
			//逐次処理
			switch(aData & 0b1111)
			{
				case LOGCMD_EXIT:
					break;
				case LOGCMD_POWER:
					aParam1 = (aData >> 24) & 0b1;
					callback_dscommand("PW", aParam1, 0, 0, 2);
					
					aCmd.function = "PW";
					aCmd.arg0 = aParam1;
					aCmd.arg1 = 0;
					aCmd.arg2 = 0;
					break;
				
				case LOGCMD_CHANGELOCOADDRESS:
					u2_locaddress_playing = (aData >> 4) & 0xFFFF;
					break;
				
				case LOGCMD_SPEED:
					aParam1 = (aData >> 16) & 0xFFFF;
					callback_dscommand("SP", u2_locaddress_playing + OFFSET_DCCADDRESS, aParam1, 2, 2);
					
					aCmd.function = "SP";
					aCmd.arg0 = u2_locaddress_playing + OFFSET_DCCADDRESS;
					aCmd.arg1 = aParam1;
					aCmd.arg2 = 0;
					break;
				
				case LOGCMD_FUNCTION:
					aParam1 = (aData >> 16) & 0xFF;
					aParam2 = (aData >> 24) & 0b1;
					callback_dscommand("FN", u2_locaddress_playing + OFFSET_DCCADDRESS, aParam1, aParam2, 2);
					
					aCmd.function = "FN";
					aCmd.arg0 = u2_locaddress_playing + OFFSET_DCCADDRESS;
					aCmd.arg1 = aParam1;
					aCmd.arg2 = aParam2;
					break;
				
				case LOGCMD_DIRECTION:
					aParam2 = (aData >> 24) & 0b1;//FWD=0, REV=1
					callback_dscommand("DI", u2_locaddress_playing + OFFSET_DCCADDRESS,  (aParam2 == 1) ? 2 : 1, 0, 2);
					
					aCmd.function = "DI";
					aCmd.arg0 = u2_locaddress_playing + OFFSET_DCCADDRESS;
					aCmd.arg1 = aParam2;
					aCmd.arg2 = 0;
					break;
				
				case LOGCMD_ACCESSORY:
					aParam1 = (aData >> 4) & 0x7FF;
					aParam2 = (aData >> 24) & 0b1;
					callback_dscommand("TO", aParam1 + OFFSET_DCCACCADDRESS + 1, aParam2, 0, 2);
					
					aCmd.function = "TO";
					aCmd.arg0 = aParam1 + OFFSET_DCCACCADDRESS + 1;
					aCmd.arg1 = aParam2;
					aCmd.arg2 = 0;
					break;
				
				case LOGCMD_WAIT:
					u4_wait_playing = ((aData >> 4) & 0x7FFF) + to_ms_since_boot(get_absolute_time());
					break;
				case LOGCMD_LOOP:
					break;
				case LOGCMD_JUMP:
					break;
				case LOGCMD_IF:
					break;
				case LOGCMD_NEXT:
					break;
				
			}
			
			//カウンタをインクリメント
			u2_playcounter++;			
			
		}
	}
	
	return aCmd;
	
}

void LogManager::StartLog()
{
	
	u1_state = LOGSTATE_LOG;
	
}

void LogManager::InfoDialogLog()
{
	
	u1_state = LOGSTATE_INFO;
	
}

void LogManager::IdleLog()
{
	
	u1_state = LOGSTATE_IDLE;
	
}

void LogManager::PlayLog()
{
	
	u1_state = LOGSTATE_PLAY;
	
	//ウェイト初期化
	u4_wait_playing = 0;
	
	u2_playcounter = 0;
}


void LogManager::EndLog()
{
	
	//終了時のウェイトを記録する
	SetWait();

	//アイドル状態へ
	u1_state = LOGSTATE_IDLE;
	
}

uint8_t LogManager::GetState()
{
	return u1_state;
}

uint32_t LogManager::GetLog(uint16_t u2_row)
{
	uint32_t u4_pack = 0;
	
	
	if( u2_row < u2_logcounter)
	{
		u4_pack = u4_logdata[u2_row];
		
	}
	
	return u4_pack;
}


void LogManager::Stack(uint32_t u4_packed)
{
	
	if( u1_state == LOGSTATE_LOG)
	{
	
		u4_logdata[u2_logcounter] = u4_packed;
		u2_logcounter++;
	}
	
}

void LogManager::SetLocoSpeed(uint16_t address, uint16_t inSpeed)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	//ロコアドレスセット
	SetLocoAddress(address);
	
	u4_pack += LOGCMD_SPEED;//4bit
	u4_pack += 0 << 4;
	u4_pack += (inSpeed) << 16;
	
	
	//Serial.print("SPD ");
	//Serial.println(inSpeed);
	
	
	SetWait();
	Stack(u4_pack);
	
}

void LogManager::SetLocoAddress(uint16_t address)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	if( address != u2_locaddress)
	{
		u4_pack += LOGCMD_CHANGELOCOADDRESS;//4bit
		u4_pack += (address & 0xFFFF) << 4;
		
		SetWait();
		Stack(u4_pack);
		
		u2_locaddress = address;
	}
}

void LogManager::SetLocoFunction(uint16_t address, uint8_t inFunction, uint8_t inPower)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	//ロコアドレスセット
	SetLocoAddress(address);
	
	u4_pack += LOGCMD_FUNCTION;//4bit
	u4_pack += 0 << 4;
	u4_pack += (inFunction & 0xFF) << 16;
	u4_pack += (inPower & 0x1) << 24;
	
	SetWait();
	Stack(u4_pack);

}

/// @brief 
/// @param address ロコアドレス
/// @param inDirection 進行方向 FWD=0, REV=1
void LogManager::SetLocoDirection(uint16_t address, uint8_t inDirection)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	//ロコアドレスセット
	SetLocoAddress(address);
	
	u4_pack += LOGCMD_DIRECTION;//4bit
	u4_pack += 0 << 4;
	u4_pack += 0 << 16;
	u4_pack += ((inDirection >= 2) ? 1 : 0) << 24;
	
	SetWait();
	Stack(u4_pack);

}

void LogManager::SetTurnout(uint16_t address, uint8_t inSwitch)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	u4_pack += LOGCMD_ACCESSORY;//4bit
	u4_pack += (address & 0x7FF) << 4;
	u4_pack += 0 << 16;
	u4_pack += (inSwitch & 0x1) << 24;
	
	SetWait();
	Stack(u4_pack);

}

void LogManager::SetPower(uint8_t power)
{
	uint32_t u4_pack = 0;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	u4_pack += LOGCMD_POWER;//4bit
	u4_pack += 0 << 4;
	u4_pack += 0 << 16;
	u4_pack += (power & 0x1) << 24;
	
	SetWait();
	Stack(u4_pack);

}

void LogManager::SetWait()
{
	uint32_t u4_pack = 0;
	uint32_t u4_wait = to_ms_since_boot(get_absolute_time()) - u4_currenttime;
	
	//状態チェック
	if( u1_state != LOGSTATE_LOG)
	{
		return;
	}
	
	if( u4_wait > 0xFFFFFFF)
	{
		//Limit 268435.455‬sec
		u4_wait = 0xFFFFFFF;
	}
	
	if( (u4_wait > 250) && (u2_logcounter > 0))
	{
		u4_pack += LOGCMD_WAIT;//4bit
		u4_pack += u4_wait << 4;
		Stack(u4_pack);
		
	}
	
	//次のウェイト用
	u4_currenttime = to_ms_since_boot(get_absolute_time());
	
}


uint16_t LogManager::GetLocoAddress()
{
	return u2_locaddress;
	
}


uint16_t LogManager::GetLength()
{
	return u2_logcounter;
	
}

uint16_t LogManager::GetCurrentRow()
{
	return u2_playcounter;
	
}



