//////////////////////////////////////////////////////////////////////////////////
// 
// Desktop Station Log Manager
// Copyright(C)'2024- DesktopStation
// 
// Not use for commercial use
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//


#ifndef LOGMANAGER_LIB_H
#define LOGMANAGER_LIB_H

#include <Arduino.h>

#define		OFFSET_DCCADDRESS		49152
#define		OFFSET_DCCACCADDRESS	14335


#define		LOGSIZE			1024

/* コマンド */
#define		LOGCMD_EXIT					0
#define		LOGCMD_POWER				1
#define		LOGCMD_CHANGELOCOADDRESS	2
#define		LOGCMD_SPEED				3
#define		LOGCMD_FUNCTION				4
#define		LOGCMD_DIRECTION			5
#define		LOGCMD_ACCESSORY			6
#define		LOGCMD_WAIT					7
#define		LOGCMD_LOOP					8
#define		LOGCMD_JUMP					9
#define		LOGCMD_IF					10
#define		LOGCMD_NEXT					11

/* 状態 */
#define		LOGSTATE_IDLE				0
#define		LOGSTATE_LOG				1
#define		LOGSTATE_PLAY				2
#define		LOGSTATE_INFO				3

typedef struct
{
	String function;
	word arg0;
	word arg1;
	word arg2;
} LOG_CMD;

typedef void (*SetFunctionDSCommand) (const String inFunction, const word inArg1, const word inArg2, const word inArg3, const word inArg4);
typedef void (*SetFunctionDSCore) (const uint32_t inArg1, const uint32_t inArg2);


/* Defined functions */

class LogManager
{
	private:
		uint32_t u4_logdata[LOGSIZE];
		uint16_t u2_logcounter;
		uint16_t u2_playcounter;
		uint32_t u4_currenttime;
	
		uint16_t u2_locaddress;
		uint16_t u2_locaddress_playing;
		uint32_t u4_wait_playing;
		
		uint8_t u1_state;
		
		
	public:

		uint32_t u4_FlagFunction;
	
	
		LogManager();
	
		void Clear();
		bool Exist();
		void Load();
		void Save();
		void Reset();
		LOG_CMD Main();
		void Stack(uint32_t u4_packed);
		void SetLocoSpeed(uint16_t address, uint16_t inSpeed);
		void SetLocoAddress(uint16_t address);
		void SetLocoDirection(uint16_t address, uint8_t inDirection);
		void SetLocoFunction(uint16_t address, uint8_t inFunction, uint8_t inPower);
		void SetTurnout(uint16_t address, uint8_t inSwitch);
		void SetPower(uint8_t power);
		void SetWait();
		void StartLog();
		void PlayLog();
		void EndLog();
		void IdleLog();
		void InfoDialogLog();
		LOG_CMD RunInterpreter();
	
		uint32_t GetLog(uint16_t u2_row);
		uint16_t GetLocoAddress();
		uint16_t GetLength();
		uint16_t GetCurrentRow();
		uint8_t GetState();
	
	
		SetFunctionDSCommand callback_dscommand;
		SetFunctionDSCore callback_dscore;
		
};

#endif
