//////////////////////////////////////////////////////////////////////////////////
// 
// Desktop Station UIManager
// Copyright(C)'2023- DesktopStation, Nagoden, Fujigaya2, Maison de DCC
// 
// Not use for commercial use
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//


#ifndef DSUIMANAGER_LIB_H
#define DSUIMANAGER_LIB_H


#include <Arduino.h>
#include "ParamManager.h"
#include "LogManager.h"


#define		UI_BUTTON_1			2
#define		UI_BUTTON_2			4
#define		UI_BUTTON_3			8
#define		UI_BUTTON_4			64
#define		UI_BUTTON_5			128
#define		UI_BUTTON_6			256
#define		UI_BUTTON_7			2048
#define		UI_BUTTON_8			4096
#define		UI_BUTTON_9			8192
#define		UI_BUTTON_0			32
#define		UI_BUTTON_POWER		16
#define		UI_BUTTON_DIR		16384
#define		UI_BUTTON_FNC		512
#define		UI_BUTTON_ENTER		1024
#define		UI_BUTTON_MODE		1


#define		UI_STATE_IDLE		0
#define		UI_STATE_REFRESH	1

#define		UI_MODE_LOGO		0
#define		UI_MODE_HOME		1
#define		UI_MODE_LOCO		2
#define		UI_MODE_ACC			3
#define		UI_MODE_CVREAD		4
#define		UI_MODE_CVWRITE		5
#define		UI_MODE_ANALOG		6
#define		UI_MODE_LOCOCHK		7
#define		UI_MODE_STATUS		8
#define		UI_MODE_DEVICEINFO	9
#define		UI_MODE_LOCOEDIT	10
#define		UI_MODE_CONTROLINFO	11
#define		UI_MODE_HEIM		12
#define		UI_MODE_LOGPLAY		13
#define		UI_MODE_CONFIG		99
#define		UI_MODE_QRURL		100
#define		UI_MODE_ERROR		255

#define		BUTTON_NONE			0
#define		BUTTON_PRESS		1
#define		BUTTON_PRESSING		2
#define		BUTTON_LEAVED		4

#define		LEN_LOCODATA		32
#define		LEN_ACCDATA			127 //255

#define		MSGPOSY_ERROR		4

typedef void (*SetFunctionDSCommand) (const String inFunction, const word inArg1, const word inArg2, const word inArg3, const word inArg4);
typedef void (*SetFunctionDSCore) (const uint32_t inArg1, const uint32_t inArg2);

typedef struct
{
	word mAddress;	
	word mSpeed;
	uint8_t mDir;
	uint32_t mFuncBuf;
} UI_LOCODATA;

/* Defined functions */

class UIManager
{
	private:

		UI_LOCODATA locoDatas[LEN_LOCODATA];
		uint32_t u32_button_data;
		int32_t s32_speed_ref;
		uint32_t u32_button_data_last;
		int32_t s32_speed_ref_last;
	
		//電流センサバッファ
		int32_t offset_current;
		
		//表示状態
		uint8_t state;
		
		//表示モード
		uint8_t mode;
		uint8_t mode_last;
		
		//ファンクションシフト状態
		int32_t fnc_shift;
		
		//線路状態
		word dcc_power;
		
		//アドレス
		int32_t loco_address;
		word loco_speed;
		word loco_dir;
		uint32_t loco_func;
		int32_t acc_shift;
		word acc_address;
		uint8_t u8_acc_data[LEN_ACCDATA + 1];
		
		//CV関連
		word cv_address;
		word cv_value;
		word cv1_value;
		word cv8_value;
		word cv17_value;
		word cv18_value;
		word cv29_value;
		int32_t loco_address_edited;//アドレス編集用Locアドレス
		
		//アナログPWM運転用
		word pwmana_speed;
		word pwmana_lightoffset;
		word pwmana_dir;
		uint8_t pwmana_output;
		
		//LOG用
		uint16_t u2_logDisplayRow;
		

		//Matrixボタン関連
		uint8_t gMButton_state;
		uint8_t gMButton_buttonrow[3];
		
		//ロゴ自動オフカウンタ
		uint8_t count_logo_off;

		//アニメーション
		uint8_t cycle_anime;
		uint8_t cycle_anime_dir;
		
		//画面ごとの状態変数
		uint8_t state_loco;
		uint8_t state_acc;
		uint8_t state_home;
		uint8_t state_cvr;
		uint8_t state_cvw;
		uint8_t state_addr;
		uint8_t state_cfg;
		uint8_t state_analog;
		uint8_t state_editaddr;
		uint8_t state_config;
		uint8_t state_info;
		uint8_t state_qrcode;

		//MODE MENU 画面番号
		uint8_t number_screen_home;

		//クライアント存在チェック
		word flag_wifi_client;
		word flag_wifi_client_last;
		word flag_usb_pc;
		word flag_usb_pc_last;

		//SSIDデータポインタ
		char *pSSID;
		char *pPASSWORD;
		char *pIPHost;

		//ロングアドレス閾値
		uint8_t Threshold_LongAddr;

		//VR取得バッファ
		uint32_t buf_VR;

		//更新バッファ
		uint32_t flags_update_display_inv;
		
		//エラー
		word error_code;
		
		//お知らせ用
		uint32_t time_playing;// 遊んだ時間(0.5sec)
		uint32_t time_wifi_connect;// WiFi接続時間(0.1sec)
		uint32_t count_pushing;// ボタンを押した回数
		uint32_t watt_playing;// 積算消費電力(mW)
		word peak_current;// 最大ピーク電流(mA)
		
		//Log管理
		LogManager logMan;
		
		//パラメータ管理
		ParamManager paramMan;
		
		void printString(char *s, int x, int y);
		void printStringNOT(char *s, int x, int y);
		void printStringNumWordNOT(int a, int x, int y);
		void printStringNumWord(int a, int x, int y);
		void printStringNumWordsNOT(int a, int x, int y);
		void printStringNumWords(int a, int x, int y);
		void drawSpeedBar(uint8_t x, uint8_t y, uint8_t speed, uint8_t inNot);
		void clearDisplay();
		void UpdateDisplay_inv_power();
		void UpdateDisplay_inv_dir();
		void UpdateDisplay_inv_speed();
		void UpdateDisplay_inv_address(uint8_t inEditMode=0);
		void UpdateDisplay_inv_acc_address(uint8_t inEditMode=0);
		void UpdateDisplay_inv_function();
		void UpdateDisplay_inv_sensor_volt();
		void UpdateDisplay_inv_sensor_amp();		
		void UpdateDisplay_inv_wifi();
		void UpdateDisplay_inv_usbpc();
		void UpdateDisplay_inv_accessory();
		void UpdateDisplay_inv_cvread(bool fin_read);
		void UpdateDisplay_inv_cvwrite(bool fin_write);
		void UpdateDisplay_inv_checkaddress();
		void UpdateDisplay_inv_addredit();
		void UpdateDisplay_inv_controllist();
		void UpdateDisplay_inv_config();
		void UpdateDisplay_inv_analog();
		void UpdateDisplay_inv_ModeMenu();
		void UpdateDisplay_inv_animation_page();
		void UpdateDisplay_inv_animation_power();
		void UpdateDisplay_inv_animation_dir();
		void UpdateDisplay_inv_logo();
		void UpdateDisplay_inv_info();
		void UpdateDisplay_inv_info_interval();
		void UpdateDisplay_inv_log();
		void UpdateDisplay_inv_QRCode();
		void UpdateDisplay_msgdlgr(const char *inText, int x, int y);
		void UpdateDisplay_msgdlgInfo(const char *inText, int x, int y);
		bool CheckStatusFunction(int32_t inNo);
		void PressedFunction(int32_t inNo);
		void PressedAddress(int32_t inNo);
		void PressedAccessory(int32_t inNo);
		void PressedAddressAcc(int32_t inNo);
		void PressedCVNo(int32_t inNo);
		void PressedCVNoWrite(int32_t inNo);
		void ConvertToZChar(int32_t inNumbers, int32_t inLength, char inData[]);
		uint8_t GetAccessoryStatus(int32_t inAddress);
		void DrawForm();
    void UpdateCVAddressEdit(int32_t inAddressNum);
    void refresh();
    const char* Lang(int ID);
	
	public:
		UIManager();
	
		void Init();
		void Clear();
		void Process();
	

		/* OLED */
		OzOLED LCD;
		
		void IntervalVRSpeed();
		int32_t GetVRSpeed();
		int32_t GetVdc();
		int32_t GetCurrent();
		
		void InitMatrixButton();
		uint32_t GetMatrixButton();
		bool IntervalMatrixButton();

		void MonitorUserAction();
		void MonitorUserAction_LOCO();
		void MonitorUserAction_ACC();
		void MonitorUserAction_CVREAD();
		void MonitorUserAction_CVWRITE();
		void MonitorUserAction_ADDRREAD();
		void MonitorUserAction_ANALOG();
		void MonitorUserAction_EditLocoAddr();
		void MonitorUserAction_Config();
		void MonitorUserAction_Log();
		void ProcessInterval();
		void UpdateDisplay();
		uint8_t CheckButton(uint32_t inButton);
		void SetResultCVValue(word inCVValue);
		void NotifyError(word inErrorCode);
		void NotifyWiFiClient();
		void NotifyUSBPC();
		void NotifyDCCPower(word inPower);
		void NotifyDCCDirection(word inAddress, word inDirection);
		void NotifyDCCSpeed(word inAddress, word inSpeed);
		void NotifyDCCFunction(word inAddress, word inFuncNo, word inPower);
		void NotifyAccessory(int32_t inAddr, word inPower);

		//Log専用通知処理
		void NotifyLogDCCPower(word inPower);
		void NotifyLogDCCDirection(word inAddress, word inDirection);
		void NotifyLogDCCSpeed(word inAddress, word inSpeed);
		void NotifyLogDCCFunction(word inAddress, word inFuncNo, word inPower);
		void NotifyLogAccessory(int32_t inAddr, word inPower);

		void UpdateLocoDataSpeed(word inAddress, word inSpeed);
		void UpdateLocoDataFunction(word inAddress, word inFuncNo, word inPower);
		void UpdateLocoDataDirection(word inAddress, word inDirection);

		void StoreLocoData(UI_LOCODATA inLocData);
		bool GetLocoData(UI_LOCODATA *opData);

		void RegisterWifi(char *inpSSID, char *inpPASS);
	
		void ApplyCommandUI(String function, word arg0, word arg1, word arg2);
	
		SetFunctionDSCommand callback_dscommand;
		SetFunctionDSCore callback_dscore;

		int32_t WiFiStatus;
		bool UserWebApp;
		
};

#endif
